Index: lib/libpam/modules/pam_ssh/pam_ssh.8
===================================================================
RCS file: /usr/CVS-Repository/src/lib/libpam/modules/pam_ssh/pam_ssh.8,v
retrieving revision 1.13
diff -u -r1.13 pam_ssh.8
--- lib/libpam/modules/pam_ssh/pam_ssh.8	2 Jul 2004 23:52:18 -0000	1.13
+++ lib/libpam/modules/pam_ssh/pam_ssh.8	19 May 2005 00:13:20 -0000
@@ -93,6 +93,10 @@
 option,
 except that if the previously obtained password fails,
 the user is prompted for another password.
+.It Cm nullok
+If this option is set then pam_ssh will consider keys with
+no passphrase. Normally it will ignore those keys for the
+purposes of authentication.
 .El
 .Ss SSH Session Management Module
 The
Index: lib/libpam/modules/pam_ssh/pam_ssh.c
===================================================================
RCS file: /usr/CVS-Repository/src/lib/libpam/modules/pam_ssh/pam_ssh.c,v
retrieving revision 1.40
diff -u -r1.40 pam_ssh.c
--- lib/libpam/modules/pam_ssh/pam_ssh.c	10 Feb 2004 10:13:21 -0000	1.40
+++ lib/libpam/modules/pam_ssh/pam_ssh.c	19 May 2005 00:17:49 -0000
@@ -87,7 +87,7 @@
  * struct pam_ssh_key containing the key and its comment.
  */
 static struct pam_ssh_key *
-pam_ssh_load_key(const char *dir, const char *kfn, const char *passphrase)
+pam_ssh_load_key(const char *dir, const char *kfn, const char *passphrase, int nullok)
 {
 	struct pam_ssh_key *psk;
 	char fn[PATH_MAX];
@@ -97,6 +97,22 @@
 	if (snprintf(fn, sizeof(fn), "%s/%s", dir, kfn) > (int)sizeof(fn))
 		return (NULL);
 	comment = NULL;
+
+	/* Test if the key is loadable with no passphrase */
+	if ((key = key_load_private(fn, "", &comment)) != NULL) {
+		free(comment);
+		key_free(key);
+	}
+
+	/* If nullok is set check that the passphrase we got from the user is
+	 * null since SSH will ignore the passphrase we pass in if there is no
+	 * passphrase on the key so if we don't check that what is passed in
+	 * is empty the user will be able to login with *any* password(!)
+	 */
+	if (!(nullok && passphrase[0] == '\0') && key != NULL) {
+		openpam_log(PAM_LOG_NOTICE, "null passphrase was found, refusing to continue\n");
+		return (NULL);
+	}
 	key = key_load_private(fn, passphrase, &comment);
 	if (key == NULL) {
 		openpam_log(PAM_LOG_DEBUG, "failed to load key from %s\n", fn);
@@ -136,11 +152,16 @@
 	const char **kfn, *passphrase, *user;
 	struct passwd *pwd;
 	struct pam_ssh_key *psk;
-	int nkeys, pam_err, pass;
+	int nkeys, pam_err, pass, nullok;
 
 	/* PEM is not loaded by default */
 	OpenSSL_add_all_algorithms();
 
+	if (openpam_get_option(pamh, "nullok") != NULL)
+		nullok = 1;
+	else
+		nullok = 0;
+
 	/* get user name and home directory */
 	pam_err = pam_get_user(pamh, &user, NULL);
 	if (pam_err != PAM_SUCCESS)
@@ -170,7 +191,7 @@
 	/* try to load keys from all keyfiles we know of */
 	nkeys = 0;
 	for (kfn = pam_ssh_keyfiles; *kfn != NULL; ++kfn) {
-		psk = pam_ssh_load_key(pwd->pw_dir, *kfn, passphrase);
+		psk = pam_ssh_load_key(pwd->pw_dir, *kfn, passphrase, nullok);
 		if (psk != NULL) {
 			pam_set_data(pamh, *kfn, psk, pam_ssh_free_key);
 			++nkeys;
